<?php

/**
 * @file
 * Contains \Drupal\selectize\Element\Selectize.
 */

namespace Drupal\selectize\Element;

use \Drupal\Core\Render\Element\FormElement;
use \Drupal\Core\Render\Element;
use \Drupal\Core\Form\FormStateInterface;
use \Drupal\Component\Utility\Unicode as Unicode;

/**
 * Provides a selectized form element.
 *
 * @FormElement("selectize")
 */
class Selectize extends FormElement {
  /**
   * {@inheritdoc}
   */
  public function getInfo() {
    $class = get_class($this);

    return array(
      '#input' => TRUE,
      '#multiple' => FALSE,
      '#autocomplete_route_name' => FALSE,
      '#process' => array(
        array($class, 'processSelectize'),
      ),
      '#pre_render' => array(
        array($class, 'preRenderSelectize'),
      ),
      '#theme' => 'select',
      '#theme_wrappers' => array('form_element'),
      '#options' => array(),
    );
  }

  /**
   * Prepares a #type 'selectize' render element for input.html.twig.
   *
   * @param array $element
   *   An associative array containing the properties of the element.
   *   Properties used: #title, #value, #description, #size, #maxlength,
   *   #placeholder, #required, #attributes.
   *
   * @return array
   *   The $element with prepared variables ready for input.html.twig.
   */
  public static function preRenderSelectize($element) {
    Element::setAttributes($element, array('id', 'name', 'size'));
    static::setAttributes($element, array('form-select'));
    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public static function processSelectize(&$element, FormStateInterface $form_state, &$complete_form) {
    if (isset($element['#settings'])) {
      $complete_form['#attached']['drupalSettings']['selectize'][$element['#id']] = json_encode($element['#settings']);

      // if drag_drop plugin is requested, we need to load the sortable plugin.
      if (isset($element['#settings']['plugins']) && in_array('drag_drop', $element['#settings']['plugins'])) {
        $complete_form['#attached']['library'][] = 'core/jquery.ui.sortable';
      }

      // inject the selectize library and CSS assets.
      $complete_form['#attached']['library'][] = 'selectize/selectize.core';
      $complete_form['#attached']['library'][] = 'selectize/selectize.drupal';
    }

    return $element;
  }

  /**
   * Return default settings for Selectize. Pass in values to override defaults.
   * @param $values
   * @return array
   */
  public static function settings(array $values = array()) {
    $settings = array(
      'create' => FALSE,
      'sortField' => 'text',
      'plugins' => NULL,
      'highlight' => TRUE,
      'maxItems' => 10,
      'delimiter' => NULL,
      'persist' => FALSE,
    );

    return array_merge($settings, $values);
  }
}